@@ -90,10 +90,12 @@ static const R_CMethodDef CEntries[]  = {
 };
 
 static R_CallMethodDef CallEntries[] = {
-    CALLDEF(R_rho_inf, 2),
+    CALLDEF(R_rho_inf, 2), // -> lmrob.c
     CALLDEF(R_psifun, 4),
     CALLDEF(R_chifun, 4),
     CALLDEF(R_wgtfun, 3),
+    CALLDEF(R_wgt_flex, 3), // -> rob-utils.c
+    CALLDEF(R_rowMedians, 5),// -> rowMedians.c [Biobase also has rowQ for quantiles]
     {NULL, NULL, 0}
 };
 
